<?php

namespace Drupal\basicshib;

use Drupal\basicshib\Plugin\BasicShibPluginManager;
use Drupal\basicshib\Plugin\GrouperPluginInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\user\UserInterface;

/**
 *
 */
class AuthorizationHandler implements AuthorizationHandlerInterface {

//  use GrouperHelperTrait;

  /**
   * Grouper plugin instance variable.
   *
   * @var GrouperPluginInterface
   */
  private $grouper;

  /**
   * @var \Drupal\basicshib\SessionTracker
   */
  private SessionTracker $session_tracker;

  /**
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected RequestStack $requestStack;

  protected \Drupal\Core\Config\ImmutableConfig $basicShibSettings;

  protected \Drupal\Core\Config\ImmutableConfig $grouperSettings;

  /**
   * AuthorizationHandler constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   * @param \Drupal\basicshib\Plugin\BasicShibPluginManager $grouper_plugin_manager
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function __construct(ConfigFactoryInterface $config_factory,
                              RequestStack $request_stack,
                              BasicShibPluginManager $grouper_plugin_manager) {

    $this->requestStack = $request_stack;
    if ($this->requestStack->getCurrentRequest()->hasSession()) {
      $this->session_tracker = new SessionTracker(
        $this->requestStack->getCurrentRequest()->getSession()
      );
    }

    $this->basicShibSettings = $config_factory->get('basicshib.settings');
    $this->grouperSettings = $config_factory->get('basicshib.grouper_settings');
    $plugins = $this->basicShibSettings->get('plugins');

    // Create instance of grouper plugin.
    $this->grouper = $grouper_plugin_manager
      ->createInstance($plugins['grouper']);
  }

  /**
   * @param \Drupal\user\UserInterface $current_account
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function authorize(UserInterface $current_account) {
    $userGroups = [];
    // Logs the HTTP request that is user-specific
    // at the time of current login session.


//    $userGroups = explode(';', $this->requestStack->server->get('isMemberOf'));
      $userGroups = [
        'admin-grouper-policy',
        'staff-grouper-policy',
        'student-grouper-policy',
      ];

    $previousRoles = $current_account->getRoles();
    // Get array of the roles a user is currently granted from GrouperPluginDefault.
    $groupRoleMap = $this->grouper->getMap($userGroups);
    // Compare The roles from Grouper to the roles currently granted in Drupal and add any new roles granted.
    $newRoles = array_diff($groupRoleMap, $previousRoles);
    foreach ($newRoles as &$role) {
      $current_account->addRole($role);
    }
    // Compare The roles from Grouper to the roles currently granted in Drupal and remove any roles now blocked.
    $delRoles = array_diff($previousRoles, $groupRoleMap);
    foreach ($delRoles as &$role) {
      switch ($role) {

        case 'administrator':
          // If administrator role removed from user and admin role removal
          // allowed, set in the Basicshib settings form, remove the role from
          // the user.
//          if ($this->grouperSettings->get('role_management')['admin_removal']) {
//            $current_account->removeRole($role);
//          }
          break;

        default:
          $current_account->removeRole($role);
      }
    }

    $current_account->save();
  }

}
