<?php

declare(strict_types=1);

namespace Drupal\basicshib\Entity;

use Drupal\Core\Config\Entity\ConfigEntityBase;
use Drupal\basicshib\AuthenticationInterface;

/**
 * Defines the authentication entity type.
 *
 * @ConfigEntityType(
 *   id = "authentication",
 *   label = @Translation("authentication"),
 *   label_collection = @Translation("authentications"),
 *   label_singular = @Translation("authentication"),
 *   label_plural = @Translation("authentications"),
 *   label_count = @PluralTranslation(
 *     singular = "@count authentication",
 *     plural = "@count authentications",
 *   ),
 *   handlers = {
 *     "list_builder" = "Drupal\basicshib\AuthenticationListBuilder",
 *     "form" = {
 *       "add" = "Drupal\basicshib\Form\AuthenticationForm",
 *       "edit" = "Drupal\basicshib\Form\AuthenticationForm",
 *       "delete" = "Drupal\Core\Entity\EntityDeleteForm",
 *     },
 *   },
 *   config_prefix = "authentication",
 *   admin_permission = "administer authentication",
 *   links = {
 *     "collection" = "/admin/structure/authentication",
 *     "add-form" = "/admin/structure/authentication/add",
 *     "edit-form" = "/admin/structure/authentication/{authentication}",
 *     "delete-form" = "/admin/structure/authentication/{authentication}/delete",
 *   },
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label",
 *     "uuid" = "uuid",
 *   },
 *   config_export = {
 *     "id",
 *     "label",
 *     "shibboleth_default_login",
 *     "default_post_login_redirect_path",
 *     "login_handler",
 *     "logout_handler",
 *     "name_attribute",
 *     "mail_attribute",
 *     "session_id_attribute",
 *     "login_link_label",
 *     "external_redirect_error",
 *     "generic_login_error",
 *     "account_blocked_error",
 *     "login_disallowed_error",
 *     "user_creation_not_allowed_error",
 *     "grouper_authorization_enabled",
 *     "remove_administrator_role",
 *     "remove_authenticated_role",
 *     "description",
 *   },
 * )
 */
final class Authentication extends ConfigEntityBase implements AuthenticationInterface {

  /**
   * The authentication ID.
   */
  protected string $id;

  /**
   * The authentication label.
   */
  protected string $label;

  /**
   * Enable Shibboleth as the default login.
   */
  protected bool $shibboleth_default_login;

  /**
   * The default_post_login_redirect_path label.
   */
  protected string $default_post_login_redirect_path;

  /**
   * The login_handler label.
   */
  protected string $login_handler;

  /**
   * The logout_handler label.
   */
  protected string $logout_handler;

  /**
   * The name_attribute label.
   */
  protected string $name_attribute;

  /**
   * The mail_attribute label.
   */
  protected string $mail_attribute;

  /**
   * The session_id attribute label.
   */
  protected string $session_id_attribute;

  /**
   * The login_link_label label.
   */
  protected string $login_link_label;

  /**
   * The external_redirect_error label.
   */
  protected string $external_redirect_error;

  /**
   * The generic_login_error label.
   */
  protected string $generic_login_error;

  /**
   * The account_blocked_error label.
   */
  protected string $account_blocked_error;

  /**
   * The login_disallowed_error label.
   */
  protected string $login_disallowed_error;

  /**
   * The user_creation_not_allowed_error label.
   */
  protected string $user_creation_not_allowed_error;

  /**
   * Enable Grouper for login authorization.
   */
  protected bool $grouper_authorization_enabled;

  /**
   * Remove administrator role on logout.
   */
  protected bool $remove_administrator_role;

  /**
   * Remove authenticated role on logout.
   */
  protected bool $remove_authenticated_role;

  /**
   * The authentication description.
   */
  protected string $description;

}
