<?php

declare(strict_types=1);

namespace Drupal\basicshib\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\basicshib\Entity\AuthFilter;

/**
 * auth-filter form.
 */
final class AuthFilterForm extends EntityForm {

  // In most instances the default values will be used so load them. They selected if the
  // 'Use Default values button is selected  on the 'Auth-filter List form', else the default
  // values are set as 'empty'.

  public function form(array $form, FormStateInterface $form_state): array {

    $values = [];
    $defaults = \Drupal::request()->query->get('defaults');
    if ($defaults == TRUE) {
      $config = \Drupal::config('basicshib.auth_filter');
      $values = [
        'label' => $config->get('label'),
        'allow' => $config->get('allow'),
        'error' => $config->get('error'),
        'description' => $config->get('description'),
      ];
    } else {
      $values = [
        'label' => $this->entity->get('label'),
        'allow' => $this->entity->get('allow'),
        'error' => $this->entity->get('error'),
        'description' => $this->entity->get('description'),
      ];
    }
    // Load the 'basicshib.settings' configuration object.
    $config = \Drupal::config('basicshib.auth_filter');

    $form = parent::form($form, $form_state);

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $config->get('label'),
      '#required' => TRUE,
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $this->entity->id(),
      '#machine_name' => [
        'exists' => [AuthFilter::class, 'load'],
      ],
      '#disabled' => !$this->entity->isNew(),
    ];

    $form['status'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enabled'),
      '#default_value' => $this->entity->status(),
    ];

    $form['create'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Create'),
    ];

    $form['create']['allow'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Allow'),
      '#maxlength' => 255,
      '#default_value' => $config->get('create')['allow'],
    ];

    $form['create']['error'] = [
      '#type' => 'textfield',
      '#title' => $this->t('error'),
      '#maxlength' => 255,
      '#default_value' => $config->get('create')['error'],
      '#required' => TRUE,
    ];

    $form['description'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Description'),
      '#default_value' => $config->get('description'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state): int {
    $result = parent::save($form, $form_state);
    $message_args = ['%label' => $this->entity->label()];
    $this->messenger()->addStatus(
      match($result) {
        \SAVED_NEW => $this->t('Created new auth-filter %label.', $message_args),
        \SAVED_UPDATED => $this->t('Updated auth-filter %label.', $message_args),
      }
    );
    $form_state->setRedirectUrl($this->entity->toUrl('collection'));
    return $result;
  }

}
