<?php

declare(strict_types=1);

namespace Drupal\basicshib\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\basicshib\Entity\Authentication;
Use \Drupal\Core\Routing;

/**
 * authentication form.
 */
final class AuthenticationForm extends EntityForm {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state): array {

    // in most instances the default values will be used so load them. They selected if the
    // 'Use Default values button is selected  on the 'Authentication List form', else the default
    // values are set as 'empty'.

    $values = [];
    $defaults = \Drupal::request()->query->get('defaults');
    if ($defaults == TRUE) {
      $config = \Drupal::config('basicshib.settings');
      $values = [
        'label' => $config->get('label'),
        'default_post_login_redirect_path' => $config->get('default_post_login_redirect_path'),
        'login_handler' => $config->get('handlers')['login'],
        'logout_handler' => $config->get('handlers')['logout'],
        'name_attribute' => $config->get('attribute_map')['key']['name'],
        'mail_attribute' => $config->get('attribute_map')['key']['mail'],
        'session_id_attribute' => $config->get('attribute_map')['key']['session_id'],
        'login_link_label' => $config->get('login_link_label'),
        'external_redirect_error' => $config->get('messages')['external_redirect_error'],
        'generic_login_error' => $config->get('messages')['generic_login_error'],
        'account_blocked_error' => $config->get('messages')['account_blocked_error'],
        'login_disallowed_error' => $config->get('messages')['login_disallowed_error'],
        'user_creation_not_allowed_error' => $config->get('messages')['user_creation_not_allowed_error'],
        'description' => $config->get('description'),
      ];
    } else {
      $values = [
        'label' => $this->entity->get('label'),
        'default_post_login_redirect_path' => $this->entity->get('default_post_login_redirect_path'),
        'login_handler' => $this->entity->get('login_handler'),
        'logout_handler' => $this->entity->get('logout_handler'),
        'name_attribute' => $this->entity->get('name_attribute'),
        'mail_attribute' => $this->entity->get('mail_attribute'),
        'session_id_attribute' => $this->entity->get('session_id_attribute'),
        'login_link_label' => $this->entity->get('login_link_label'),
        'external_redirect_error' => $this->entity->get('external_redirect_error'),
        'generic_login_error' => $this->entity->get('generic_login_error'),
        'account_blocked_error' => $this->entity->get('account_blocked_error'),
        'login_disallowed_error' => $this->entity->get('login_disallowed_error'),
        'user_creation_not_allowed_error' => $this->entity->get('user_creation_not_allowed_error'),
        'description' => $this->entity->get('description'),
      ];
    }

    $form = parent::form($form, $form_state);

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $values['label'],
      '#required' => TRUE,
    ];

    $form['shibboleth_default_login'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Shibboleth default login'),
      '#default_value' => $this->entity->get('shibboleth_default_login'),
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $this->entity->id(),
      '#machine_name' => [
        'exists' => [Authentication::class, 'load'],
      ],
      '#disabled' => !$this->entity->isNew(),
    ];

    $form['status'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enabled'),
      '#default_value' => $this->entity->status(),
    ];

    $form['default_post_login_redirect_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Default post-login redirect path'),
      '#description' => $this->t('The location that the user will be redirected to after shibboleth login, when no location is specified by the return URL.'),
      '#maxlength' => 64,
      '#size' => 64,
      '#default_value' => $values['default_post_login_redirect_path'],
    ];

    $form['login_handler'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Login handler'),
      '#maxlength' => 64,
      '#size' => 64,
      '#default_value' => $values['login_handler'],
    ];

    $form['logout_handler'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Logout handler'),
      '#maxlength' => 64,
      '#size' => 64,
      '#default_value' => $values['logout_handler'],
    ];

    $form['attributes'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Attributes'),
    ];

    $form['attributes']['name_attribute'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Username attribute'),
      '#maxlength' => 64,
      '#size' => 64,
      '#default_value' => $values['name_attribute'],
    ];

    $form['attributes']['mail_attribute'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Mail attribute'),
      '#maxlength' => 64,
      '#size' => 64,
      '#default_value' => $values['mail_attribute'],
    ];

    $form['attributes']['session_id_attribute'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Session ID attribute'),
      '#maxlength' => 64,
      '#size' => 64,
      '#default_value' => $values['session_id_attribute'],
    ];

    $form['ui'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('User interface'),
    ];

    $form['ui']['login_link_label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Login link label'),
      '#description' => $this->t('The label of the login link'),
      '#maxlength' => 64,
      '#size' => 64,
      '#default_value' => $values['login_link_label'],
    ];

    $form['ui']['external_redirect_error'] = [
      '#type' => 'textarea',
      '#title' => $this->t('External redirect error message'),
      '#description' => $this->t('The message to display when an attempt to redirect to an external URL occurs. This may be malicious so contact information for reporting the incident is good to have here.'),
      '#maxlength' => 1024,
      '#rows' => 3,
      '#default_value' => $values['external_redirect_error'],
    ];

    $form['ui']['generic_login_error'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Generic login error'),
      '#description' => $this->t('The message to display when login fails due to an error which cannot be disclosed to the browser.'),
      '#maxlength' => 1024,
      '#rows' => 3,
      '#default_value' => $values['generic_login_error'],
    ];

    $form['ui']['account_blocked_error'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Account blocked error'),
      '#description' => $this->t('The message to display when a user can be authenticated with Shibboleth, but is blocked by Drupal.'),
      '#maxlength' => 1024,
      '#rows' => 3,
      '#default_value' => $values['account_blocked_error'],
    ];

    $form['ui']['login_disallowed_error'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Login disallowed error'),
      '#description' => $this->t('The message to display when a user login is disallowed for a reason other than being blocked.'),
      '#maxlength' => 1024,
      '#rows' => 3,
      '#default_value' => $values['login_disallowed_error'],
    ];

    $form['ui']['user_creation_not_allowed_error'] = [
      '#type' => 'textarea',
      '#title' => $this->t('User creation disallowed error'),
      '#description' => $this->t('The message to display when a user cannot be created.'),
      '#maxlength' => 1024,
      '#rows' => 3,
      '#default_value' => $values['user_creation_not_allowed_error'],
    ];

    $form['grouper_authorization_enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Grouper authorization enabled'),
      '#default_value' => $this->entity->get('grouper_authorization_enabled'),
      '#attributes' => ['name' => 'grouper_authorization_enabled'],
    ];

    $form['remove_roles'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Remove roles at logout'),
      '#states' => [
        'visible' => [
          ':input[name="grouper_authorization_enabled"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['remove_roles']['remove_administrator_role'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Administrator Role'),
      '#default_value' => $this->entity->get('remove_administrator_role'),
    ];

    $form['remove_roles']['remove_authenticated_role'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Authenticated Role'),
      '#default_value' => $this->entity->get('remove_authenticated_role'),
    ];

    $form['description'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Description'),
      '#default_value' => $values['description'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state): int {
    $result = parent::save($form, $form_state);
    $message_args = ['%label' => $this->entity->label()];
    $this->messenger()->addStatus(
      match($result) {
        \SAVED_NEW => $this->t('Created new authentication %label.', $message_args),
        \SAVED_UPDATED => $this->t('Updated authentication %label.', $message_args),
      }
    );
    $form_state->setRedirectUrl($this->entity->toUrl('collection'));
    return $result;
  }

}
