<?php

declare(strict_types=1);

namespace Drupal\basicshib;

use Drupal;
use Drupal\user\Entity\Role;

/**
 *
 */
trait GrouperHelperTrait {

  /**
   * Queries available Drupal roles from the specific website.
   * returns an array with the Drupal roles excluding roles passed in the ignoreRoles array.
   *
   * @return array
   */
  public static function getDrupalRoles(array $ignoreRoles = []): array {
    $definedRoles = array_keys(\Drupal\user\Entity\Role::loadMultiple());
    $validRoles = [];
    foreach ($definedRoles as $role) {
      if (!in_array ($role, $ignoreRoles)) {
        $validRoles[] = $role;
      }
    }

    return $validRoles;
  }

    /**
   * Queries available Grouper Policies from the specific website.
   * returns an array of all the policies.
   *
   * @return array
   */
  public static function getGrouperPolicies(): array {
    $grouperPolicies = \Drupal::config('basicshib.Grouper Policies.drupal_admin');

    return $grouperPolicies;
  }

  /**
   * Returns a user attribute specified by a string.
   *
   * @param string
   *   denotes the desired attribute name
   *
   * @return string
   */
  protected function getUserAttribute($attribute_name): ?string {
    $req = Drupal::request();
    return $req->server->get($attribute_name);
  }

}
