<?php

namespace Drupal\basicshib\Plugin\Derivative;

use Drupal\Component\Plugin\Derivative\DeriverBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Plugin\Discovery\ContainerDeriverInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

class DynamicLocalTasks extends DeriverBase implements ContainerDeriverInterface {

  /**
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  public function __construct(ConfigFactoryInterface $config_factory) {
    $this->configFactory = $config_factory;
  }

  public static function create(ContainerInterface $container, $base_plugin_id): DynamicLocalTasks {
    return new static(
      $container->get('config.factory')
    );
  }

  /**
   * Hide or display the grouper tab depending on if the 'Enable Grouper'
   * checkbox is set.
   */
  public function getDerivativeDefinitions($base_plugin_definition): array {
    $baseRoute = NULL;
    $settings = $this->configFactory->get('basicshib.settings');

    if ($settings->get('plugin_enabled')['grouper_enabled']) {
      $baseRoute = 'basicshib.core_settings_form';
    }
    $this->derivatives['entity.grouper_edit_form'] = $base_plugin_definition;
    $this->derivatives['entity.grouper_edit_form']['title'] = "Grouper Role settings";
    $this->derivatives['entity.grouper_edit_form']['route_name'] = 'entity.grouper_role.collection';
    $this->derivatives['entity.grouper_edit_form']['description'] = 'Configure the Grouper plugin';
    $this->derivatives['entity.grouper_edit_form']['base_route'] = $baseRoute;
    return parent::getDerivativeDefinitions($base_plugin_definition);
  }

}
